package cd.casic.module.machine.dal.mysql;
import cd.casic.framework.commons.pojo.PageResult;
import cd.casic.framework.mybatis.core.mapper.BaseMapperX;
import cd.casic.framework.mybatis.core.query.LambdaQueryWrapperX;
import cd.casic.module.machine.controller.vo.MachineEnvVO;
import cd.casic.module.machine.controller.vo.MachineProxyVO;
import cd.casic.module.machine.dal.dataobject.MachineEnvDO;
import cd.casic.module.machine.dal.dataobject.MachineProxyDO;
import cd.casic.module.machine.enums.MachineProxyStatus;
import org.apache.ibatis.annotations.Mapper;

import java.util.List;

import static com.baomidou.mybatisplus.extension.toolkit.Db.listObjs;

/**
 * 机器代理Mapper接口
 */
@Mapper
public interface MachineProxyMapper extends BaseMapperX<MachineProxyDO> {



    default PageResult<MachineProxyDO> selectPage(MachineProxyVO machineProxyVO) {
        return selectPage(machineProxyVO, new LambdaQueryWrapperX<MachineProxyDO>()
                .eqIfPresent(MachineProxyDO::getHostIp, machineProxyVO.getHostIp())
                .eqIfPresent(MachineProxyDO::getProxyType, machineProxyVO.getProxyType())
                .eqIfPresent(MachineProxyDO::getStatus, machineProxyVO.getStatus())
                .likeIfPresent(MachineProxyDO::getDescription, machineProxyVO.getDescription())
        );
    }
    default List<Long> getOnlineProxyListInIds(List<Long> ids) {
        LambdaQueryWrapperX<MachineProxyDO> queryWrapper = new LambdaQueryWrapperX<>();

        // 1. 状态为在线（eqIfPresent 确保状态非空时拼接条件）
        queryWrapper.eqIfPresent(MachineProxyDO::getStatus, MachineProxyStatus.ONLINE.getCode());

        // 2. 传入的 ID 列表非空时，添加 IN 条件（使用 inIfPresent 自动过滤空值）
        queryWrapper.inIfPresent(MachineProxyDO::getId, ids);

        // 4. 仅查询 ID 字段，提升性能
        return listObjs(queryWrapper.select(MachineProxyDO::getId));
    }
}