//package cd.casic.module.machine.handler;
//import cd.casic.module.machine.utils.AliYunOssClient;
//import cd.casic.module.machine.dal.dataobject.MachineInfo;
//import cd.casic.module.machine.dal.dataobject.SecretKey;
//import cd.casic.module.machine.enums.ConnectionStatus;
//import cd.casic.module.machine.service.SecretKeyService;
//import com.jcraft.jsch.*;
//import jakarta.annotation.Resource;
//import lombok.extern.slf4j.Slf4j;
//import org.springframework.stereotype.Component;
//import org.springframework.util.StreamUtils;
//import org.springframework.util.StringUtils;
//import java.io.*;
//import java.nio.charset.StandardCharsets;
//import java.util.Objects;
//import java.util.Properties;
//import java.util.concurrent.atomic.AtomicBoolean;
//
///**
// * 优化后的SSH连接会话类
// */
//@Slf4j
//@Component
//public class ConnectionSession implements AutoCloseable {
//    @Resource
//    SecretKeyService secretKeyService;
//
//    @Resource
//    AliYunOssClient aliYunOssClient;
//
//    private MachineInfo machineInfo;
//    private Session sshSession;
//    private ConnectionStatus status = ConnectionStatus.DISCONNECTED;
//    private final AtomicBoolean isExecuting = new AtomicBoolean(false);
//
//    // todo连接配置常量
//    private static final int CONNECTION_TIMEOUT = 5000; // 连接超时时间(毫秒)
//    private static final int COMMAND_TIMEOUT = 30000; // 命令执行超时时间(毫秒)
//    private static final int RETRY_COUNT = 3; // 重试次数
//    private static final int RETRY_DELAY = 1000; // 重试间隔(毫秒)
//
//    public ConnectionSession() {
//
//    }
//
//    // 使用setter注入MachineInfo
//    public void setMachineInfo(MachineInfo machineInfo) {
//        this.machineInfo = Objects.requireNonNull(machineInfo, "MachineInfo cannot be null");
//        log.debug("MachineInfo 已注入: {}", machineInfo.getHostIp());
//    }
//
//
//
//    /**
//     * 建立SSH连接，支持重试机制
//     */
//    public synchronized void connect() throws JSchException {
//        if (status == ConnectionStatus.CONNECTED) {
//            log.debug("Already connected to {}", machineInfo.getHostIp());
//            return;
//        }
//
//        status = ConnectionStatus.CONNECTING;
//        JSchException lastException = null;
//
//        for (int attempt = 1; attempt <= RETRY_COUNT; attempt++) {
//            try {
//                doConnect();
//                status = ConnectionStatus.CONNECTED;
//                log.info("SSH connection established successfully to {} (attempt {}/{})",
//                    machineInfo.getHostIp(), attempt, RETRY_COUNT);
//                return;
//            } catch (JSchException e) {
//                lastException = e;
//                status = ConnectionStatus.CONNECTION_ERROR;
//                log.error("SSH connection attempt {}/{} failed: {}",
//                    attempt, RETRY_COUNT, e.getMessage());
//
//                // 认证失败直接退出，无需重试
//                if (e.getMessage().contains("Auth fail")) {
//                    status = ConnectionStatus.AUTH_FAILED;
//                    throw e;
//                }
//
//                // 重试前等待
//                if (attempt < RETRY_COUNT) {
//                    try {
//                        Thread.sleep(RETRY_DELAY);
//                    } catch (InterruptedException ie) {
//                        Thread.currentThread().interrupt();
//                        throw new JSchException("Connection attempt interrupted", ie);
//                    }
//                }
//            } catch (IOException e) {
//                throw new RuntimeException(e);
//            }
//        }
//
//        // 所有重试都失败
//        throw new JSchException("Failed to connect after " + RETRY_COUNT + " attempts", lastException);
//    }
//
//    /**
//     * 实际执行连接逻辑
//     */
//    private void doConnect() throws JSchException, IOException {
//        JSch jsch = new JSch();
//
//        // 配置认证方式
//        configureAuthentication(jsch);
//
//        // 创建SSH会话
//        sshSession = jsch.getSession(
//            machineInfo.getUsername(),
//            machineInfo.getHostIp(),
//            machineInfo.getSshPort() != null ? machineInfo.getSshPort() : 22
//        );
//
//        // 配置连接参数
//        configureSession(sshSession);
//
//        // 建立连接
//        sshSession.connect(CONNECTION_TIMEOUT);
//    }
//
//    /**
//     * 配置认证方式（密码或密钥）
//     */
//    private void configureAuthentication(JSch jsch) throws JSchException {
//        if (machineInfo.getAuthenticationTypeCode() == 2) {
//            // 密钥认证
//            if (machineInfo.getSecretKeyId() == null) {
//                throw new JSchException("Secret key ID is required for key-based authentication");
//            }
//
//            String privateKeyContent = getPrivateKeyContent(machineInfo.getSecretKeyId());
//            // 验证私钥格式
//            if (!privateKeyContent.startsWith("-----BEGIN")) {
//                throw new JSchException("Invalid private key format. Expected OpenSSH format.");
//            }
//
//            try {
//                // 尝试加载私钥
//                jsch.addIdentity(
//                        machineInfo.getName(),
//                        privateKeyContent.getBytes(StandardCharsets.UTF_8),
//                        null,
//                        null
//                );
//                log.info("Private key loaded successfully for {}", machineInfo.getHostIp());
//            } catch (JSchException e) {
//                log.error("Failed to load private key: {}", e.getMessage());
//                throw e;
//            }
//        } else if (machineInfo.getAuthenticationTypeCode() == 1) {
//            // 密码认证
//            if (StringUtils.isEmpty(machineInfo.getPassword())) {
//                throw new JSchException("Password is required for password-based authentication");
//            }
//        } else {
//            throw new JSchException("Unsupported authentication type: " + machineInfo.getAuthenticationType());
//        }
//    }
//
//    /**
//     * 配置SSH会话参数（安全增强）
//     */
//    private void configureSession(Session session) {
//        Properties config = new Properties();
//
//        // 安全增强：默认验证主机密钥
//        if (isTrustedEnvironment()) {
//            log.warn("Running in trusted environment - disabling strict host key checking for {}",
//                machineInfo.getHostIp());
//            config.put("StrictHostKeyChecking", "no");
//        } else {
//            config.put("StrictHostKeyChecking", "yes");
//            // 可选：配置已知主机文件路径
//            //直接配置阿里云密钥地址
//            config.put("UserKnownHostsFile", secretKeyService.getById(machineInfo.getSecretKeyId()).getPath());
//        }
//
//        // 其他安全配置
//        config.put("PreferredAuthentications", "publicKey,password,keyboard-interactive");
//        config.put("ServerAliveInterval", "30"); // 每30秒发送一次心跳
//        config.put("ServerAliveCountMax", "3"); // 允许3次心跳失败
//
//        session.setConfig(config);
//    }
//
//    /**
//     * 判断是否为可信环境（生产环境应返回false）
//     */
//    private boolean isTrustedEnvironment() {
//        // todo实际项目中应基于配置或环境变量判断
//        return System.getProperty("environment", "production").equalsIgnoreCase("development");
//    }
//
//    @Override
//    public synchronized void close() {
//        disconnect();
//    }
//
//    public synchronized void disconnect() {
//        if (sshSession != null && sshSession.isConnected()) {
//            try {
//                sshSession.disconnect();
//                log.info("SSH connection closed: {}", machineInfo.getHostIp());
//            } catch (Exception e) {
//                log.error("Error closing SSH session: {}", e.getMessage());
//            }
//        }
//        status = ConnectionStatus.DISCONNECTED;
//    }
//
//    /**
//     * 执行远程命令，支持超时和中断处理
//     */
//    public String executeCommand(String command) throws JSchException, IOException {
//        if (!isConnected()) {
//            throw new IllegalStateException("Session is not connected");
//        }
//
//        if (!isExecuting.compareAndSet(false, true)) {
//            throw new IllegalStateException("Another command is already executing");
//        }
//
//        Channel channel = null;
//        InputStream inputStream = null;
//        ByteArrayOutputStream outputStream = null;
//
//        try {
//            channel = createExecChannel(command);
//            inputStream = channel.getInputStream();
//            outputStream = new ByteArrayOutputStream();
//
//            // 连接通道并设置超时
//            channel.connect(COMMAND_TIMEOUT);
//
//            // 读取命令输出
//            return readCommandOutput(inputStream, outputStream, channel);
//        } catch (InterruptedException e) {
//            Thread.currentThread().interrupt();
//            throw new IOException("Command execution interrupted", e);
//        } finally {
//            // 释放资源
//            closeResources(channel, inputStream, outputStream);
//            isExecuting.set(false);
//        }
//    }
//
//    /**
//     * 创建并配置命令执行通道
//     */
//    private Channel createExecChannel(String command) throws JSchException {
//        Channel channel = sshSession.openChannel("exec");
//        ((ChannelExec) channel).setCommand(command);
//
//        // 配置通道
//        channel.setInputStream(null);
//        ((ChannelExec) channel).setErrStream(new ByteArrayOutputStream()); // 捕获错误输出
//
//        return channel;
//    }
//
//    /**
//     * 读取命令输出
//     */
//    private String readCommandOutput(InputStream inputStream,
//                                     ByteArrayOutputStream outputStream,
//                                     Channel channel) throws IOException, InterruptedException {
//        byte[] buffer = new byte[1024];
//
//
//        // 使用线程中断机制实现超时控制
//        Thread readerThread = new Thread(() -> {
//            int bytesRead;
//            try {
//                while ((bytesRead = inputStream.read(buffer)) != -1) {
//                    outputStream.write(buffer, 0, bytesRead);
//                }
//            } catch (IOException e) {
//                // 通道关闭或读取异常
//                if (channel.isConnected()) {
//                    log.warn("Error reading command output: {}", e.getMessage());
//                }
//            }
//        });
//
//        readerThread.start();
//
//        // 等待命令执行完成或超时
//        readerThread.join(COMMAND_TIMEOUT);
//
//        // 如果线程仍在运行，中断并关闭通道
//        if (readerThread.isAlive()) {
//            readerThread.interrupt();
//            channel.disconnect();
//            throw new IOException("Command execution timed out after " + COMMAND_TIMEOUT + "ms");
//        }
//
//        // 等待通道完全关闭
//        while (channel.isConnected()) {
//            Thread.sleep(100);
//        }
//
//        return outputStream.toString(StandardCharsets.UTF_8);
//    }
//
//    /**
//     * 关闭资源
//     */
//    private void closeResources(Channel channel, InputStream inputStream, OutputStream outputStream) {
//        if (outputStream != null) {
//            try {
//                outputStream.close();
//            } catch (IOException e) {
//                log.warn("Error closing output stream: {}", e.getMessage());
//            }
//        }
//
//        if (inputStream != null) {
//            try {
//                inputStream.close();
//            } catch (IOException e) {
//                log.warn("Error closing input stream: {}", e.getMessage());
//            }
//        }
//
//        if (channel != null && channel.isConnected()) {
//            channel.disconnect();
//        }
//    }
//
//    /**
//     * 上传文件到远程服务器
//     */
//    public boolean uploadFile(String localFilePath, String remoteFilePath) throws IOException {
//        if (!isConnected()) {
//            throw new IllegalStateException("Cannot upload file: SSH session is not connected");
//        }
//
//        // 检查本地文件是否存在且可读
//        File localFile = new File(localFilePath);
//        if (!localFile.exists()) {
//            throw new FileNotFoundException("Local file not found: " + localFilePath);
//        }
//        if (!localFile.canRead()) {
//            throw new IOException("Cannot read local file: " + localFilePath);
//        }
//
//        ChannelSftp channel = null;
//        boolean uploadSuccess = false;
//
//        try {
//            // 创建并连接SFTP通道，设置超时
//            channel = (ChannelSftp) sshSession.openChannel("sftp");
//            channel.connect(CONNECTION_TIMEOUT);
//
//            // 确保目标目录存在
//            createRemoteDirectoryIfNotExists(channel, getParentDirectory(remoteFilePath));
//
//            // 使用更健壮的上传方式
//            channel.put(
//                new FileInputStream(localFile),
//                remoteFilePath,
//                new ProgressMonitorAdapter(localFile.length()),
//                ChannelSftp.OVERWRITE
//            );
//
//            uploadSuccess = true;
//            log.info("File uploaded successfully: {} -> {}", localFilePath, remoteFilePath);
//            return true;
//
//        } catch (SftpException e) {
//            log.error("SFTP error during file upload ({} -> {}): {}",
//                localFilePath, remoteFilePath, e.getMessage(), e);
//            throw new IOException("SFTP error: " + e.getMessage(), e);
//        } catch (FileNotFoundException e) {
//            log.error("Local file not found during upload: {}", localFilePath, e);
//            throw e;
//        } catch (IOException e) {
//            log.error("IO error during file upload: {}", e.getMessage(), e);
//            throw e;
//        } catch (Exception e) {
//            log.error("Unexpected error during file upload: {}", e.getMessage(), e);
//            throw new IOException("Unexpected error: " + e.getMessage(), e);
//        } finally {
//            // 确保通道始终被关闭
//            disconnectChannel(channel);
//
////            // 如果上传失败，尝试删除不完整的文件
////            if (!uploadSuccess && remoteFilePath != null && !remoteFilePath.isEmpty()) {
////                tryDeleteIncompleteFile(remoteFilePath);
////            }
//        }
//    }
//
//    public boolean downloadFile(String remoteFilePath, String localFilePath) throws IOException {
//        if (!isConnected()) {
//            throw new IllegalStateException("Cannot download file: SSH session is not connected");
//        }
//
//        // 检查本地目录是否存在且可写
//        File localFile = new File(localFilePath);
//        File parentDir = localFile.getParentFile();
//        if (parentDir != null && !parentDir.exists()) {
//            if (!parentDir.mkdirs()) {
//                throw new IOException("Failed to create local directory: " + parentDir.getAbsolutePath());
//            }
//        }
//        if (parentDir != null && !parentDir.canWrite()) {
//            throw new IOException("Cannot write to local directory: " + parentDir.getAbsolutePath());
//        }
//
//        ChannelSftp channel = null;
//        boolean downloadSuccess = false;
//        File tempFile = null;
//
//        try {
//            // 创建并连接SFTP通道，设置超时
//            channel = (ChannelSftp) sshSession.openChannel("sftp");
//            channel.connect(CONNECTION_TIMEOUT);
//
//            // 检查远程文件是否存在
//            SftpATTRS attrs = channel.stat(remoteFilePath);
//            long fileSize = attrs.getSize();
//
//            // 使用临时文件避免部分下载覆盖完整文件
//            tempFile = new File(localFilePath + ".part");
//
//            // 执行下载并监控进度
//            channel.get(
//                remoteFilePath,
//                new FileOutputStream(tempFile).toString(),
//                new ProgressMonitorAdapter(fileSize),
//                ChannelSftp.OVERWRITE
//            );
//
//            // 验证下载完整性
//            if (tempFile.length() != fileSize) {
//                throw new IOException("Download incomplete: expected " + fileSize +
//                    " bytes, but got " + tempFile.length() + " bytes");
//            }
//
//            // 重命名临时文件为目标文件（原子操作）
//            if (!tempFile.renameTo(localFile)) {
//                throw new IOException("Failed to rename temporary file to: " + localFilePath);
//            }
//
//            downloadSuccess = true;
//            log.info("File downloaded successfully: {} -> {}", remoteFilePath, localFilePath);
//            return true;
//
//        } catch (SftpException e) {
//            log.error("SFTP error during file download ({} -> {}): {}",
//                remoteFilePath, localFilePath, e.getMessage(), e);
//            if (e.id == ChannelSftp.SSH_FX_NO_SUCH_FILE) {
//                throw new FileNotFoundException("Remote file not found: " + remoteFilePath);
//            }
//            throw new IOException("SFTP error: " + e.getMessage(), e);
//        } catch (IOException e) {
//            log.error("IO error during file download: {}", e.getMessage(), e);
//            throw e;
//        } catch (Exception e) {
//            log.error("Unexpected error during file download: {}", e.getMessage(), e);
//            throw new IOException("Unexpected error: " + e.getMessage(), e);
//        } finally {
//            // 确保通道始终被关闭
//            disconnectChannel(channel);
//
//            // 如果下载失败，清理临时文件
//            if (!downloadSuccess && tempFile != null && tempFile.exists()) {
//                if (tempFile.delete()) {
//                    log.debug("Deleted incomplete temporary file: {}", tempFile.getAbsolutePath());
//                } else {
//                    log.warn("Failed to delete incomplete temporary file: {}", tempFile.getAbsolutePath());
//                }
//            }
//        }
//    }
//
//    // 创建远程目录（如果不存在）
//    private void createRemoteDirectoryIfNotExists(ChannelSftp channel, String directory) throws SftpException {
//        if (directory == null || directory.isEmpty() || directory.equals("/")) {
//            return;
//        }
//
//        try {
//            channel.stat(directory);
//        } catch (SftpException e) {
//            // 目录不存在，尝试创建
//            createRemoteDirectoryIfNotExists(channel, getParentDirectory(directory));
//            channel.mkdir(directory);
//            log.debug("Created remote directory: {}", directory);
//        }
//    }
//
//    // 获取路径的父目录
//    private String getParentDirectory(String path) {
//        int lastSlash = path.lastIndexOf('/');
//        return lastSlash > 0 ? path.substring(0, lastSlash) : "";
//    }
//
//    // 断开SFTP通道
//    private void disconnectChannel(Channel channel) {
//        if (channel != null && channel.isConnected()) {
//            try {
//                channel.disconnect();
//                log.debug("SFTP channel disconnected");
//            } catch (Exception e) {
//                log.warn("Error disconnecting SFTP channel: {}", e.getMessage());
//            }
//        }
//    }
//
//    // 尝试删除不完整的文件
//    private void tryDeleteIncompleteFile(String remoteFilePath) {
//        ChannelSftp channel = null;
//        try {
//            channel = (ChannelSftp) sshSession.openChannel("sftp");
//            channel.connect(CONNECTION_TIMEOUT);
//            channel.rm(remoteFilePath);
//            log.info("Deleted incomplete file: {}", remoteFilePath);
//        } catch (Exception e) {
//            log.warn("Failed to delete incomplete file {}: {}", remoteFilePath, e.getMessage());
//        } finally {
//            disconnectChannel(channel);
//        }
//    }
//
//    // 增强的进度监控器
//    private static class ProgressMonitorAdapter implements SftpProgressMonitor {
//        private final long totalBytes;
//        private long bytesWritten = 0;
//        private int lastProgress = 0;
//        private final long startTime = System.currentTimeMillis();
//
//        public ProgressMonitorAdapter(long totalBytes) {
//            this.totalBytes = totalBytes;
//        }
//
//        @Override
//        public boolean count(long count) {
//            bytesWritten += count;
//
//            // 计算进度百分比
//            int progress = (int) ((bytesWritten * 100) / totalBytes);
//
//            // 每10%或每秒更新一次日志
//            long elapsedTime = System.currentTimeMillis() - startTime;
//            if (progress - lastProgress >= 10 || elapsedTime >= 1000) {
//                double speed = bytesWritten / (elapsedTime / 1000.0);
//                String speedStr = formatTransferSpeed(speed);
//
//                log.debug("Upload progress: {}% ({}/{} bytes, {})",
//                    progress, bytesWritten, totalBytes, speedStr);
//                lastProgress = progress;
//            }
//
//            return true; // 返回true继续传输，返回false中断传输
//        }
//
//        @Override
//        public void end() {
//            long elapsedTime = System.currentTimeMillis() - startTime;
//            double speed = totalBytes / (elapsedTime / 1000.0);
//            String speedStr = formatTransferSpeed(speed);
//
//            log.info("Upload completed: {} bytes in {} ms (avg speed: {})",
//                totalBytes, elapsedTime, speedStr);
//        }
//
//        @Override
//        public void init(int op, String src, String dest, long max) {
//            log.info("Starting upload: {} -> {} ({} bytes)", src, dest, max);
//        }
//
//        // 格式化传输速度
//        private String formatTransferSpeed(double bytesPerSecond) {
//            String[] units = {"B/s", "KB/s", "MB/s", "GB/s"};
//            int unitIndex = 0;
//
//            while (bytesPerSecond >= 1024 && unitIndex < units.length - 1) {
//                bytesPerSecond /= 1024;
//                unitIndex++;
//            }
//
//            return String.format("%.2f %s", bytesPerSecond, units[unitIndex]);
//        }
//    }
//
//
//    /**
//     * 检查连接状态
//     */
//    public ConnectionStatus getStatus() {
//        if (status == ConnectionStatus.CONNECTED &&
//            (sshSession == null || !sshSession.isConnected())) {
//            status = ConnectionStatus.DISCONNECTED;
//        }
//        return status;
//    }
//
//    /**
//     * 检查是否已连接
//     */
//    public boolean isConnected() {
//        return status == ConnectionStatus.CONNECTED &&
//            sshSession != null &&
//            sshSession.isConnected();
//    }
//
//
//    private String getPrivateKeyContent(Long secretKeyId) {
//        if (secretKeyId == null) {
//            return null;
//        }
//        SecretKey secretKey = secretKeyService.getById(secretKeyId);
//        byte[] content;
//        try {
//            content = aliYunOssClient.getContent(secretKey.getPath().substring(secretKey.getPath().lastIndexOf("/") + 1));
//        } catch (Exception e) {
//            throw new RuntimeException(e);
//        }
//
//        //改为S3FileClient读取
//        InputStream read = new ByteArrayInputStream(content);
//
//        try {
//            return StreamUtils.copyToString(read, StandardCharsets.UTF_8);
//        } catch (IOException e) {
//            log.error("读取私钥文件失败", e);
//            throw new RuntimeException(e);
//        }
//
//
//    }
//}
