package cd.casic.module.machine.service;

import cd.casic.framework.commons.pojo.PageResult;
import cd.casic.module.machine.controller.vo.MachineInfoDto;
import cd.casic.module.machine.dal.dataobject.MachineInfo;
import cd.casic.module.machine.enums.ConnectionStatus;
import com.baomidou.mybatisplus.extension.service.IService;

import java.util.Map;

public interface MachineInfoService  {
    boolean addMachineInfo(MachineInfoDto MachineInfoDto);

    PageResult<MachineInfoDto> listMachineInfo(MachineInfoDto MachineInfoDto);

    boolean updateMachineInfo(MachineInfoDto machineInfoDto);

    boolean updateStatus(MachineInfoDto machineInfoDto);

    boolean bindingSecretKey(MachineInfoDto machineInfoDto);

    void deleteList(String machineInfoIds);

    void deleteMachineInfo(Long machineInfoId);


    /**
     * 测试机器连接
     *
     * @param id 机器id
     * @return 连接是否成功
     */
    boolean testConnection(Long id);

    /**
     * 获取机器连接状态
     *
     * @param machineName 机器名称
     * @return 连接状态
     */
    ConnectionStatus getConnectionStatus(String machineName);

    /**
     * 获取所有连接状态
     *
     * @return 机器名称到连接状态的映射
     */
    Map<String, ConnectionStatus> getAllConnectionStatus();

    /**
     * 建立机器连接
     *
     * @param machineInfo 机器信息
     * @return 连接会话ID
     */
    String connect(MachineInfo machineInfo);

    /**
     * 断开机器连接
     *
     * @param sessionId 会话ID
     * @return 操作结果
     */
    boolean disconnect(String sessionId);

    /**
     * 执行远程命令
     *
     * @param sessionId 会话ID
     * @param command   命令
     * @return 命令执行结果
     */
    String executeCommand(String sessionId, String command);

    /**
     * 上传文件到远程机器
     *
     * @param sessionId      会话ID
     * @param localFilePath  本地文件路径
     * @param remoteFilePath 远程文件路径
     * @return 操作结果
     */
    boolean uploadFile(String sessionId, String localFilePath, String remoteFilePath);

    /**
     * 从远程机器下载文件
     *
     * @param sessionId      会话ID
     * @param remoteFilePath 远程文件路径
     * @param localFilePath  本地文件路径
     * @return 操作结果
     */
    boolean downloadFile(String sessionId, String remoteFilePath, String localFilePath);
}
