package cd.casic.module.machine.utils;


import cd.casic.module.machine.enums.CodeEnum;

public class EnumUtils {
    /**
     * 根据code和枚举类型获取对应的枚举值
     *
     * @param code      枚举的code值
     * @param enumClass 实现了CodeEnum接口的枚举类
     * @param <T>       枚举类型
     * @return 对应的枚举值，若未找到则返回null
     */
    public static <T extends CodeEnum> T getEnumByCode(Object code, Class<T> enumClass) {
        if (code == null || enumClass == null) {
            return null;
        }

        // 遍历枚举值查找匹配的code
        for (T enumConstant : enumClass.getEnumConstants()) {
            if (code.equals(enumConstant.getCode())) {
                return enumConstant;
            }
        }
        return null;
    }

    /**
     * 根据message和枚举类型获取对应的枚举值
     *
     * @param message   枚举的message值
     * @param enumClass 实现了CodeEnum接口的枚举类
     * @param <T>       枚举类型
     * @return 对应的枚举值，若未找到则返回null
     */
    public static <T extends CodeEnum> T getEnumByMessage(String message, Class<T> enumClass) {
        if (message == null || enumClass == null) {
            return null;
        }

        // 遍历枚举值查找匹配的message
        for (T enumConstant : enumClass.getEnumConstants()) {
            if (message.equals(enumConstant.getMessage())) {
                return enumConstant;
            }
        }
        return null;
    }
}
