//package com.casic.machine.service.impl;
//
//import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
//import com.casic.commons.exception.ServiceException;
//import com.casic.commons.utils.EnumUtils;
//import com.casic.commons.utils.PageResult;
//import com.casic.machine.dto.MachineProxyDTO;
//import com.casic.machine.entity.MachineProxy;
//import com.casic.machine.enums.MachineProxyStatus;
//import com.casic.machine.enums.MachineProxyType;
//import com.casic.machine.mapper.MachineProxyMapper;
//import com.casic.machine.service.MachineProxyService;
//import org.junit.jupiter.api.BeforeEach;
//import org.junit.jupiter.api.Test;
//import org.springframework.beans.factory.annotation.Autowired;
//import org.springframework.boot.test.context.SpringBootTest;
//import org.springframework.test.annotation.Rollback;
//import org.springframework.test.context.jdbc.Sql;
//import org.springframework.transaction.annotation.Transactional;
//
//import java.util.*;
//
//import static org.junit.jupiter.api.Assertions.*;
//
//@SpringBootTest
//@Transactional
//@Rollback(true)
//@Sql(scripts = {"classpath:sql/machine_proxy_test_data.sql"})
//public class MachineProxyServiceImplTest {
//
//    @Autowired
//    private MachineProxyService machineProxyService;
//
//    @Autowired
//    private MachineProxyMapper machineProxyMapper;
//
//    private MachineProxyDTO validProxyDTO;
//    private Long existingProxyId;
//    private Long nonExistingProxyId;
//
//    @BeforeEach
//    public void setUp() {
//        // 初始化测试数据（假设 SQL 脚本已插入一条状态为 OFFLINE 的代理）
//        existingProxyId = 1L;
//        nonExistingProxyId = 999L;
//
//        // 有效代理 DTO
//        validProxyDTO = new MachineProxyDTO();
//        validProxyDTO.setProxyType(MachineProxyType.SOCKS5.getMessage());
//        validProxyDTO.setHostIp("192.168.1.100");
//        validProxyDTO.setSshPort("22");
//        validProxyDTO.setUsername("test_user");
//        validProxyDTO.setStatus(MachineProxyStatus.ONLINE.getMessage());
//    }
//
//    // ============================== 注册代理测试 ==============================
//    @Test
//    void testRegister_ValidData_ShouldSucceed() {
//        // 执行注册
//        boolean result = machineProxyService.register(validProxyDTO);
//        assertTrue(result, "注册失败");
//
//        // 使用 Lambda 表达式查询（推荐）
//        MachineProxy proxy = machineProxyMapper.selectOne(
//                new LambdaQueryWrapper<MachineProxy>()
//                        .eq(MachineProxy::getHostIp, validProxyDTO.getHostIp())
//        );
//
//        // 断言数据存在
//        assertNotNull(proxy, "代理记录未写入数据库");
//        assertEquals(MachineProxyType.SOCKS5.getCode(), proxy.getProxyTypeCode());
//        assertEquals(MachineProxyStatus.INSTALLING.getCode(), proxy.getStatusCode());
//        assertEquals(validProxyDTO.getHostIp(), proxy.getHostIp(), "IP 地址不一致");
//    }
//
//    // ============================== 更新状态测试 ==============================
//    @Test
//    void testUpdateStatus_ExistingProxy_ShouldUpdateStatus() {
//        // 准备数据：查询现有代理（状态为 OFFLINE）
//        MachineProxy proxy = machineProxyMapper.selectById(existingProxyId);
//        assertEquals(MachineProxyStatus.OFFLINE.getCode(), proxy.getStatusCode());
//
//        // 执行状态更新为 ONLINE
//        validProxyDTO.setId(existingProxyId);
//        validProxyDTO.setStatus(MachineProxyStatus.ONLINE.getMessage());
//        boolean result = machineProxyService.updateStatus(validProxyDTO);
//
//        // 验证结果
//        assertTrue(result);
//        proxy = machineProxyMapper.selectById(existingProxyId);
//        assertEquals(MachineProxyStatus.ONLINE.getCode(), proxy.getStatusCode());
//    }
//
//    @Test
//    void testUpdateStatus_NullDto_ShouldThrowException() {
//        assertThrows(ServiceException.class, () -> {
//            machineProxyService.updateStatus(null);
//        }, "MachineProxyDTO对象为空");
//    }
//
//    // ============================== 心跳测试 ==============================
//    @Test
//    void testHeartbeat_ValidData_ShouldUpdateVersionAndStatus() {
//        // 准备数据：现有代理状态为 OFFLINE，版本为 1.0.0
//        MachineProxy proxy = machineProxyMapper.selectById(existingProxyId);
//        assertEquals("1.0.0", proxy.getVersion());
//        assertEquals(MachineProxyStatus.OFFLINE.getCode(), proxy.getStatusCode());
//
//        // 发送心跳，更新版本和状态
//        validProxyDTO.setId(existingProxyId);
//        validProxyDTO.setVersion("2.0.0");
//        validProxyDTO.setStatus(MachineProxyStatus.ONLINE.getMessage());
//        machineProxyService.heartbeat(validProxyDTO);
//
//        // 验证结果
//        proxy = machineProxyMapper.selectById(existingProxyId);
//        assertEquals("2.0.0", proxy.getVersion());
//        assertEquals(MachineProxyStatus.ONLINE.getCode(), proxy.getStatusCode());
//    }
//
//    // ============================== 状态统计测试 ==============================
//    @Test
//    void testGetStatusStatistics_ShouldReturnValidCounts() {
//        // 假设测试数据中有 OFFLINE(1)、INSTALLING(1)、ONLINE(1) 三种状态
//        Map<String, Long> stats = machineProxyService.getStatusStatistics();
//
//        // 验证统计结果
//        assertEquals(3, stats.size());
//        assertTrue(stats.containsKey(MachineProxyStatus.OFFLINE.getMessage()));
//        assertTrue(stats.containsKey(MachineProxyStatus.INSTALLING.getMessage()));
//        assertTrue(stats.containsKey(MachineProxyStatus.ONLINE.getMessage()));
//        assertEquals(1L, stats.get(MachineProxyStatus.OFFLINE.getMessage()));
//    }
//
//    // ============================== 更新配置测试 ==============================
//    @Test
//    void testUpdateConfig_ValidConfig_ShouldSucceed() {
//        // 准备数据：现有代理配置为空
//        MachineProxy proxy = machineProxyMapper.selectById(existingProxyId);
//        assertNull(proxy.getConfig());
//
//        // 更新配置
//        validProxyDTO.setId(existingProxyId);
//        validProxyDTO.setConfig("{\"port\": 8080}");
//        boolean result = machineProxyService.updateConfig(validProxyDTO);
//
//        // 验证结果
//        assertTrue(result);
//        proxy = machineProxyMapper.selectById(existingProxyId);
//        assertEquals("{\"port\": 8080}", proxy.getConfig());
//    }
//
//    // ============================== 删除代理测试 ==============================
//    @Test
//    void testDelete_OfflineProxy_ShouldDeleteSuccessfully() {
//        // 准备数据：状态为 OFFLINE 的代理 ID
//        List<Long> ids = Collections.singletonList(existingProxyId);
//        machineProxyService.delete(ids);
//
//        // 验证删除
//        MachineProxy proxy = machineProxyMapper.selectById(existingProxyId);
//        assertNull(proxy);
//    }
//
//    @Test
//    void testDelete_OnlineProxy_ShouldThrowException() {
//        // 先将代理状态改为 ONLINE
//        MachineProxy onlineProxy = new MachineProxy();
//        onlineProxy.setId(existingProxyId);
//        onlineProxy.setStatusCode(MachineProxyStatus.ONLINE.getCode());
//        machineProxyMapper.updateById(onlineProxy);
//
//        // 执行删除
//        List<Long> ids = Collections.singletonList(existingProxyId);
//        assertThrows(IllegalArgumentException.class, () -> {
//            machineProxyService.delete(ids);
//        }, "以下代理处于在线状态，无法删除: 1");
//    }
//
//    // ============================== 列表查询测试 ==============================
//    @Test
//    void testList_WithStatusFilter_ShouldReturnMatchedRecords() {
//        // 查询状态为 OFFLINE 的代理
//        MachineProxyDTO queryDto = new MachineProxyDTO();
//        queryDto.setStatus(MachineProxyStatus.OFFLINE.getMessage());
//
//        PageResult<MachineProxyDTO> pageResult = machineProxyService.list(queryDto);
//
//        // 验证结果
//        assertFalse(pageResult.getList().isEmpty());
//        pageResult.getList().forEach(dto ->
//                assertEquals(MachineProxyStatus.OFFLINE.getMessage(), dto.getStatus())
//        );
//    }
//
//    @Test
//    void testList_WithHostIpFilter_ShouldReturnMatchedRecords() {
//        // 假设测试数据中存在 host_ip 为 "192.168.1.1" 的代理
//        MachineProxyDTO queryDto = new MachineProxyDTO();
//        queryDto.setHostIp("192.168.1.1");
//
//        PageResult<MachineProxyDTO> pageResult = machineProxyService.list(queryDto);
//
//        // 验证结果
//        assertFalse(pageResult.getList().isEmpty());
//        pageResult.getList().forEach(dto ->
//                assertEquals("192.168.1.1", dto.getHostIp())
//        );
//    }
//
//    // ============================== 辅助方法测试 ==============================
//    @Test
//    void testEnumUtils_ConvertCodeToMessage() {
//        // 验证代理类型枚举转换
//        String typeMessage = EnumUtils.getEnumByCode(MachineProxyType.HTTP.getCode(), MachineProxyType.class).getMessage();
//        assertEquals("HTTP", typeMessage);
//
//        // 验证状态枚举转换
//        String statusMessage = EnumUtils.getEnumByCode(MachineProxyStatus.INSTALLING.getCode(), MachineProxyStatus.class).getMessage();
//        assertEquals("安装中", statusMessage);
//    }
//}