package cd.casic.module.process.pipeline.definition.impl;

import cd.casic.ci.commons.bean.process.definition.Pipeline;
import cd.casic.ci.commons.bean.process.definition.PipelineFollow;
import cd.casic.ci.commons.bean.process.definition.PipelineFollowQuery;
import cd.casic.ci.commons.bean.utils.PipelineUtil;
import cd.casic.framework.commons.exception.ServiceException;
import cd.casic.module.process.pipeline.definition.entity.PipelineFollowEntity;
import cd.casic.module.process.process.definition.PipelineFollowService;
import cd.casic.module.process.process.definition.dao.PipelineFollowDao;
import cd.casic.module.process.toolkit.beans.BeanMapper;
import jakarta.annotation.Resource;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.Collections;
import java.util.List;

public class PipelineFollowServiceImpl implements PipelineFollowService {
    @Resource
    PipelineFollowDao pipelineFollowDao;

    @Override
    public void updateFollow(PipelineFollow pipelineFollow) {
        Pipeline pipeline = pipelineFollow.getPipeline();
        String pipelineId = pipeline.getId();
        if (!PipelineUtil.isNoNull(pipelineId)){
            throw new ServiceException(50001,"流水线id不能为空。");
        }
//        TODO
//        String userId = LoginContext.getLoginId();
        String userId = "";
        List<PipelineFollowEntity> list =
                pipelineFollowDao.findOneUserFollowPipeline(userId, pipelineId);
        //用户为收藏该流水线
        if (list.isEmpty()){
            PipelineFollowEntity followEntity = BeanMapper.map(pipelineFollow, PipelineFollowEntity.class);
            followEntity.setUserId(userId);
            String follow = pipelineFollowDao.createFollow(followEntity);
            if (!PipelineUtil.isNoNull(follow)){
                throw new ServiceException(50001,"收藏失败");
            }
            //用户已收藏该流水线
        }else {
            deleteFollow(list.get(0).getId());
        }
    }

    @Override
    public List<PipelineFollow> findUserFollowPipeline(String userId){
        List<PipelineFollowEntity> list =
                pipelineFollowDao.findUserFollowPipeline(userId);
        return BeanMapper.mapList(list, PipelineFollow.class);
    }

    @Override
    public List<PipelineFollow> findFollowQueryList(PipelineFollowQuery followQuery){
        List<PipelineFollowEntity> list =
                pipelineFollowDao.findFollowQueryList(followQuery);
        if (list == null || list.isEmpty()){
            return Collections.emptyList();
        }
        return BeanMapper.mapList(list, PipelineFollow.class);
    }

    @Override
    public void deleteFollow(String followId) {
        pipelineFollowDao.deleteFollow(followId);
    }


    public void deletePipelineFollow(String pipelineId){
        PipelineFollowQuery followQuery = new PipelineFollowQuery();
        followQuery.setPipelineId(pipelineId);
        List<PipelineFollowEntity> list = pipelineFollowDao.findFollowQueryList(followQuery);

        for (PipelineFollowEntity pipelineFollowEntity : list) {
            deleteFollow(pipelineFollowEntity.getId());
        }

    }

    @Override
    public PipelineFollow findOneFollow(String followId) {
        return BeanMapper.map(pipelineFollowDao.findOneFollow(followId), PipelineFollow.class);
    }

    @Override
    public List<PipelineFollow> findAllFollow() {
        return BeanMapper.mapList(pipelineFollowDao.findAllFollow(), PipelineFollow.class);
    }

    @Override
    public List<PipelineFollow> findAllFollowList(List<String> idList) {
        List<PipelineFollowEntity> followList = pipelineFollowDao.findAllFollowList(idList);
        return BeanMapper.mapList(followList, PipelineFollow.class);
    }
}
