package cd.casic.module.process.pipeline.definition.impl;

import cd.casic.ci.commons.bean.process.definition.Pipeline;
import cd.casic.ci.commons.bean.process.definition.PipelineOpen;
import cd.casic.ci.commons.bean.support.count.PipelineOverview;
import cd.casic.ci.commons.bean.utils.PipelineUtil;
import cd.casic.module.process.pipeline.definition.PipelineOpenService;

import cd.casic.module.process.pipeline.definition.entity.PipelineEntity;
import cd.casic.module.process.pipeline.definition.entity.PipelineOpenEntity;
import cd.casic.module.process.pipeline.overview.service.PipelineOverviewService;
import cd.casic.module.process.process.definition.dao.PipelineDao;
import cd.casic.module.process.process.definition.dao.PipelineOpenDao;
import cd.casic.module.process.support.authority.service.PipelineAuthorityService;
import cd.casic.module.process.toolkit.beans.BeanMapper;
import cd.casic.module.process.toolkit.join.JoinTemplate;
import jakarta.annotation.Resource;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Objects;

@Service
//@Exporter
public class PipelineOpenServiceImpl implements PipelineOpenService {

    @Resource
    PipelineOpenDao pipelineOpenDao;

    @Resource
    PipelineAuthorityService authorityService;

    @Resource
    PipelineOverviewService overviewService;

    @Resource
    JoinTemplate joinTemplate;

    @Resource
    PipelineDao pipelineDao;


    @Override
    public void deleteAllOpen(String pipelineId){
        List<PipelineOpen> allOpen = findAllOpenNoQuery();
        if (allOpen.isEmpty()){
            return;
        }
        for (PipelineOpen pipelineOpen : allOpen) {
            Pipeline pipeline = pipelineOpen.getPipeline();
            if (!pipeline.getId().equals(pipelineId)){
                continue;
            }
            deleteOpen(pipelineOpen.getOpenId());
        }
    }

    @Override
    public void updatePipelineOpen(String pipelineId) {
        if (pipelineId.equals("undefined")){
            return;
        }
//        TODO
//        String userId = LoginContext.getLoginId();
        String userId = "";
        PipelineOpen pipelineOpen = new PipelineOpen();
        pipelineOpen.setPipeline(new Pipeline(pipelineId));
        pipelineOpen.setCreateTime(PipelineUtil.date(1));
        pipelineOpen.setUserId(userId);
        createOpen(pipelineOpen);
    }

    @Override
    public PipelineOpen findOneOpen(String openId) {
        PipelineOpenEntity openEntity = pipelineOpenDao.findOneOpen(openId);
        PipelineOpen pipelineOpen = BeanMapper.map(openEntity, PipelineOpen.class);
        joinTemplate.joinQuery(pipelineOpen);
        return pipelineOpen;
    }

    public PipelineOpen findOneOpenNoQuery(String openId) {
        PipelineOpenEntity openEntity = pipelineOpenDao.findOneOpen(openId);
        return BeanMapper.map(openEntity, PipelineOpen.class);
    }

    @Override
    public List<PipelineOpen> findAllOpen() {
        List<PipelineOpenEntity> allOpen = pipelineOpenDao.findAllOpen();
        if (Objects.isNull(allOpen)){
            return Collections.emptyList();
        }
        List<PipelineOpen> list = BeanMapper.mapList(allOpen, PipelineOpen.class);
        joinTemplate.joinQuery(list);
        return list;
    }

    public List<PipelineOpen> findAllOpenNoQuery() {
        List<PipelineOpenEntity> allOpen = pipelineOpenDao.findAllOpen();
        if (Objects.isNull(allOpen)){
            return Collections.emptyList();
        }
        return BeanMapper.mapList(allOpen, PipelineOpen.class);
    }

    @Override
    public List<PipelineOpen> findAllOpenList(List<String> idList) {
        List<PipelineOpenEntity> openList = pipelineOpenDao.findAllOpenList(idList);
        return BeanMapper.mapList(openList, PipelineOpen.class);
    }

    //更新最近打开
    private void updateOpen(PipelineOpen pipelineOpen) {
        pipelineOpenDao.updateOpen(BeanMapper.map(pipelineOpen, PipelineOpenEntity.class));
    }

    //删除最近打开
    private void deleteOpen(String openId) {
        pipelineOpenDao.deleteOpen(openId);
    }

    //创建最近打开
    private void createOpen(PipelineOpen pipelineOpen) {
        pipelineOpenDao.createOpen(BeanMapper.map(pipelineOpen, PipelineOpenEntity.class));
    }

    public List<String> findUserOpen(int number){
        // 获取用户流水线
//        TODO
//        String userId = LoginContext.getLoginId();
        String userId = "";
        String[] userPipeline = authorityService.findUserPipelineIdString(userId);
        if (userPipeline.length == 0){
            return Collections.emptyList();
        }

        StringBuilder builder = new StringBuilder();
        for (int i = 0; i < userPipeline.length; i++) {
            builder.append("'").append(userPipeline[i]).append("'");
            if (i != userPipeline.length-1){
                builder.append(",");
            }
        }

        List<String> pipelineIds = pipelineOpenDao.findUserOpen(userId, number,builder.toString());
        if (pipelineIds.isEmpty()){
            return Collections.emptyList();
        }
        return pipelineIds;
    }

    /**
     * 查询流水线最近打开
     * @param number 查询数量
     * @return 最近打开的流水线
     */
    @Override
    public List<PipelineOpen> findUserAllOpen(int number) {

        // 获取用户流水线
//        String userId = LoginContext.getLoginId();
        String userId = "";

        List<String> pipelineIds = findUserOpen(number);
        if (pipelineIds.isEmpty()){
            return Collections.emptyList();
        }

        List<PipelineOpen> openList = new ArrayList<>();

        for (String pipelineId : pipelineIds) {
            PipelineOpen pipelineOpen = new PipelineOpen();

            PipelineEntity pipelineEntity = pipelineDao.findPipelineById(pipelineId);
            Pipeline pipeline = BeanMapper.map(pipelineEntity, Pipeline.class);
            pipelineOpen.setPipeline(pipeline);

            Integer openNumber = pipelineOpenDao.findUserOpenNumber(userId, pipelineId);
            pipelineOpen.setNumber(openNumber);

            PipelineOverview pipelineOverview = overviewService.pipelineOverview(pipelineId);
            pipelineOpen.setPipelineExecState(pipelineOverview);

            if (Objects.isNull(pipeline) || Objects.isNull(pipeline.getName())){
                deleteOpen(pipelineOpen.getOpenId());
                continue;
            }

            openList.add(pipelineOpen);
        }
        return openList;
    }


}
